#include "../std.h"
#include <cstdio>
#include <fstream>
#include "../jobs/users.h"
#include "../jobs/states.h"
#include "../config/conf.h"
#include "../config/environment.h"
#include "../config/config_file.h"
#include "../files/info_types.h"
#include "../files/info_files.h"

#ifdef NG_WRAPPER
#include "../../userinterface/wrapper.cpp"
#endif

int main(int argc,char* argv[]) {
#ifdef NG_WRAPPER
  if(!getenv("NORDUGRID_ENVIRONMENT_SETUP")) wrapper(argv);
#endif
  int n;
  bool short_output = true;
  if(!read_env_vars()) exit(1);
  string my_username("");
  uid_t my_uid=getuid();
  JobUser *my_user = NULL;
  while((n=getopt(argc,argv,"hlu:U:")) != -1) {
    switch(n) {
      case ':': { cerr<<"Missing argument\n"; return 1; };
      case 'h': {
        cout<<"gm-jobs [-h] [-l] [-u uid] [-U name]\n";
        return 0;
      };
      case 'l': {
        short_output=false;
      }; break;
      case 'u': {
        char* p;
        my_uid=strtol(optarg,&p,0);
        if((p == optarg) || (*p != 0)) {
          cerr<<"Bad user id"<<endl; return -1;
        };
      }; break;
      case 'U': {
        {
          struct passwd pw_;
          struct passwd *pw;
          char buf[BUFSIZ];
          getpwnam_r(optarg,&pw_,buf,BUFSIZ,&pw);
          if(pw == NULL) {
            cerr<<"Unknown user: "<<optarg<<endl; return -1;
          };
          my_uid=pw->pw_uid;
        };
      }; break;
      default: break;
    };
  };
  JobUsers users;
  /* recognize itself */
  {
    struct passwd pw_;
    struct passwd *pw;
    char buf[BUFSIZ];
    getpwuid_r(my_uid,&pw_,buf,BUFSIZ,&pw);
    if(pw != NULL) { my_username=pw->pw_name; };
  };
  if(my_username.length() == 0) {
    cerr << "Can't recognize myself." << endl; return -1;
  };
  my_user = new JobUser(my_username);
  if(!configure_serviced_users(users,my_uid,my_username,*my_user)) {
    cout<<"Error processing configuration."<<endl; return -1;
  };
  if(users.size() == 0) {
    cout<<"No suitable users found in configuration."<<endl; return -1;
  };
  cout << "Looking for current jobs" << endl;
  unsigned int counters[JOB_STATE_NUM];
  unsigned int counters_pending[JOB_STATE_NUM];
  for(int i=0;i<JOB_STATE_NUM;i++) { counters[i]=0; counters_pending[i]=0; };
  unsigned int jobs_total = 0;
  for(JobUsers::iterator user = users.begin();user != users.end();++user) {
    user->get_jobs()->ScanNewJobs(false);
    for(JobsList::iterator i=user->get_jobs()->begin();i!=user->get_jobs()->end();++i) {
      cout << "Job: "<<i->get_id();
      jobs_total++;
      bool pending;
      job_state_t new_state=job_state_read_file(i->get_id(),*user,pending);
      counters[new_state]++;
      if(pending) counters_pending[new_state]++;
      if(new_state == JOB_STATE_UNDEFINED) {
        cout<<" : ERROR : Unrecognizable state"<<endl; continue;
      };
      JobLocalDescription job_desc;
      if(!job_local_read_file(i->get_id(),*user,job_desc)) {
        cout<<" : ERROR : No local information."<<endl; continue;
      };
      if(short_output) {
        cout<<" : "<<states_all[new_state].name<<" : "<<job_desc.DN<<endl;
        continue;
      };
      cout<<endl;
      cout<<"\tState: "<<states_all[new_state].name;
      if(pending) cout<<" (PENDING)";
      cout<<endl;
      cout<<"\tUser: "<<job_desc.DN<<endl;
      if(job_desc.localid.length()) cout<<"\tLRMS id: "<<job_desc.localid<<endl;
      if(job_desc.jobname.length()) cout<<"\tName: "<<job_desc.jobname<<endl;
      if(job_desc.clientname.length()) cout<<"\tFrom: "<<job_desc.clientname<<endl;
    };
  };
  cout<<"Jobs total: "<<jobs_total<<endl;
  for(int i=0;i<JOB_STATE_UNDEFINED;i++) {
    cout<<" "<<states_all[i].name<<": "<<counters[i]<<" ("<<counters_pending[i]<<")"<<endl;
  };
  return 0;
}
